import { formatToBase64 } from './photoTools';
import { DEFAULT_HAS_ERRORS_STATUS, DEFAULT_IMAGE_ROTATION_DEGREES, REJECT_FILES_REASONS } from './config';
export var filterValidFiles = function filterValidFiles(_ref) {
  var files = _ref.files,
      filesToBeFiltered = _ref.filesToBeFiltered,
      acceptedFileMaxSize = _ref.acceptedFileMaxSize,
      handlePhotosRejected = _ref.handlePhotosRejected,
      setMaxSizeError = _ref.setMaxSizeError,
      setMaxPhotosError = _ref.setMaxPhotosError,
      allowUploadDuplicatedPhotos = _ref.allowUploadDuplicatedPhotos,
      maxPhotos = _ref.maxPhotos;
  var notExcedingMaxSizeFiles = [];
  var excedingMaxSizeFiles = [];

  var notExcedingMaxSizeFilesFilter = function notExcedingMaxSizeFilesFilter(filesToFilter) {
    return filesToFilter.map(function (fileToFilter) {
      if (fileToFilter.size >= acceptedFileMaxSize) {
        excedingMaxSizeFiles.push({
          rejectedFile: fileToFilter,
          reason: "" + REJECT_FILES_REASONS.maxSize + acceptedFileMaxSize
        });
      } else {
        notExcedingMaxSizeFiles.push(fileToFilter);
      }
    });
  };

  var notRepeatedFiles = [];
  var repeatedFiles = [];

  var notRepeatedFilesFilter = function notRepeatedFilesFilter(filesToFilter) {
    return filesToFilter.map(function (fileToFilter) {
      var newFilePath = fileToFilter.path,
          newFileSize = fileToFilter.size,
          newFileLastModified = fileToFilter.lastModified;
      var isFileAlready = files.some(function (file) {
        var _file$properties = file.properties,
            path = _file$properties.path,
            size = _file$properties.size,
            lastModified = _file$properties.lastModified;
        return path === newFilePath && size === newFileSize && lastModified === newFileLastModified;
      });

      if (isFileAlready) {
        repeatedFiles.push({
          rejectedFile: fileToFilter,
          reason: REJECT_FILES_REASONS.repeated
        });
      } else {
        notRepeatedFiles.push(fileToFilter);
      }
    });
  };

  notExcedingMaxSizeFilesFilter(filesToBeFiltered);

  if (excedingMaxSizeFiles.length) {
    setMaxSizeError();
    handlePhotosRejected(excedingMaxSizeFiles);
  }

  if (allowUploadDuplicatedPhotos) {
    notRepeatedFiles.push.apply(notRepeatedFiles, notExcedingMaxSizeFiles);
  } else {
    notRepeatedFilesFilter(notExcedingMaxSizeFiles);
  }

  if (repeatedFiles.length) {
    handlePhotosRejected(repeatedFiles);
  }

  if (!notRepeatedFiles.length) return notRepeatedFiles;

  if (files.length + notRepeatedFiles.length >= maxPhotos) {
    setMaxPhotosError();
    var howManyFilesToMax = maxPhotos - files.length;
    notRepeatedFiles.splice(howManyFilesToMax - 1, notRepeatedFiles.length - howManyFilesToMax);
  }

  return notRepeatedFiles;
};
export var prepareFiles = function prepareFiles(_ref2) {
  var handlePhotosRejected = _ref2.handlePhotosRejected,
      currentFiles = _ref2.currentFiles,
      newFiles = _ref2.newFiles,
      defaultFormatToBase64Options = _ref2.defaultFormatToBase64Options,
      errorCorruptedPhotoUploadedText = _ref2.errorCorruptedPhotoUploadedText,
      setCorruptedFileError = _ref2.setCorruptedFileError,
      setFiles = _ref2.setFiles,
      setIsLoading = _ref2.setIsLoading,
      _scrollToBottom = _ref2._scrollToBottom,
      _callbackPhotosUploaded = _ref2._callbackPhotosUploaded;
  newFiles.reduce(function (accumulatorPromise, nextFile, index) {
    return accumulatorPromise.then(function () {
      return formatToBase64({
        file: nextFile,
        options: defaultFormatToBase64Options
      });
    }).then(function (_ref3) {
      var file = _ref3.file,
          blob = _ref3.blob,
          originalBase64 = _ref3.originalBase64,
          croppedBase64 = _ref3.croppedBase64,
          rotation = _ref3.rotation,
          _ref3$hasErrors = _ref3.hasErrors,
          hasErrors = _ref3$hasErrors === void 0 ? DEFAULT_HAS_ERRORS_STATUS : _ref3$hasErrors;

      if (hasErrors) {
        var errorText = errorCorruptedPhotoUploadedText.replace('%{filepath}', nextFile.path);
        handlePhotosRejected([{
          rejectedFile: nextFile,
          reason: REJECT_FILES_REASONS.loadFailed
        }]);
        setCorruptedFileError(errorText);
      } else {
        currentFiles.push({
          blob: blob,
          file: file,
          hasErrors: hasErrors,
          isModified: false,
          isNew: true,
          originalBase64: originalBase64,
          properties: {
            path: nextFile.path,
            size: nextFile.size,
            lastModified: nextFile.lastModified
          },
          preview: croppedBase64,
          rotation: rotation
        });
      }
    }).then(function () {
      setFiles([].concat(currentFiles));

      if (index >= newFiles.length - 1) {
        setIsLoading(false);

        _scrollToBottom();

        _callbackPhotosUploaded(currentFiles);
      }
    });
  }, Promise.resolve());
};
export var loadInitialPhotos = function loadInitialPhotos(_ref4) {
  var initialPhotos = _ref4.initialPhotos,
      defaultFormatToBase64Options = _ref4.defaultFormatToBase64Options,
      setInitialDownloadError = _ref4.setInitialDownloadError,
      setFiles = _ref4.setFiles,
      _callbackPhotosUploaded = _ref4._callbackPhotosUploaded,
      setIsLoading = _ref4.setIsLoading;
  var filesWithBase64 = initialPhotos.map(function (url) {
    return formatToBase64({
      url: url,
      options: defaultFormatToBase64Options
    });
  });
  Promise.all(filesWithBase64).then(function (newFiles) {
    var readyPhotos = newFiles.map(function (_ref5) {
      var blob = _ref5.blob,
          croppedBase64 = _ref5.croppedBase64,
          url = _ref5.url,
          _ref5$hasErrors = _ref5.hasErrors,
          hasErrors = _ref5$hasErrors === void 0 ? DEFAULT_HAS_ERRORS_STATUS : _ref5$hasErrors;
      return {
        blob: blob,
        url: url,
        hasErrors: hasErrors,
        originalBase64: croppedBase64,
        preview: croppedBase64,
        rotation: DEFAULT_IMAGE_ROTATION_DEGREES,
        isNew: false,
        isModified: false
      };
    });

    if (readyPhotos.some(function (photos) {
      return photos.hasErrors;
    })) {
      setInitialDownloadError();
    }

    setFiles([].concat(readyPhotos));

    _callbackPhotosUploaded(readyPhotos);

    setIsLoading(false);
  });
};